/****
 *
 * Exemplo: Comparaes
 *
 * Seo: D.12.2
 *
 ****/

#include <stdio.h> /* Funo printf() */
#include <math.h>  /* Funo fabs()   */

   /* Preciso utilizada na comparao */
   /* de valores do tipo double        */
#define DELTA 1.0E-14

/****
 *
 * ComparaDoubles(): Compara dois valores do tipo double
 *                   usando erro absoluto
 *
 * Parmetros:
 *      d1 (entrada): primeiro valor
 *      d2 (entrada): segundo valor
 *
 * Retorno: 0, se os nmeros forem considerados iguais
 *          -1, se d1 for considerado menor do que d2
 *          1, se d1 for considerado maior do que d2
 *
 ****/
int ComparaDoubles(double d1, double d2)
{
   /****************************************************/
   /*                                                  */
   /* Verifica se o valor absoluto da diferena entre  */
   /* os nmeros  menor do que ou igual  preciso    */
   /* determinada pela constante DELTA. Se for o caso, */
   /* eles so considerados iguais. Caso contrrio,    */
   /* verifica qual deles  menor ou maior.            */
   /*                                                  */
   /****************************************************/

   if (fabs(d1 - d2) <= DELTA) {
      return 0; /* d1 e d2 so considerados iguais */
   } else if (d1 < d2) {
      return -1; /* d1  menor do que d2 */
   } else {
      return 1; /* d1  maior do que d2 */
   }
}

/****
 *
 * Max(): Retorna o maior dentre dois valores do tipo double
 *
 * Parmetros:
 *      d1 (entrada): primeiro valor
 *      d2 (entrada): segundo valor
 *
 * Retorno: O maior dos parmetros
 *
 ****/
double Max(double d1, double d2)
{
   return d1 > d2 ? d1 : d2;
}

/****
 *
 * ComparaDoubles2(): Compara dois valores do tipo double
 *                    usando erro relativo (Knuth)
 *
 * Parmetros:
 *      d1 (entrada): primeiro valor
 *      d2 (entrada): segundo valor
 *
 * Retorno: 0, se os nmeros forem considerados iguais
 *          -1, se d1 for considerado menor do que d2
 *          1, se d1 for considerado maior do que d2
 *
 ****/
int ComparaDoubles2(double d1, double d2)
{
   /*                                                  */
   /* Verifica se o valor relativo da diferena entre  */
   /* os nmeros  menor do que ou igual  preciso    */
   /* determinada pela constante DELTA. Se for o caso, */
   /* eles so considerados iguais. Caso contrrio,    */
   /* verifica qual deles  menor ou maior.            */
   /*                                                  */

   if (fabs(d1 - d2) <= DELTA*Max(fabs(d1), fabs(d2))) {
      return 0; /* d1 e d2 so considerados iguais */
   } else if (d1 < d2) {
      return -1; /* d1  menor do que d2 */
   } else {
      return 1; /* d1  maior do que d2 */
   }
}

/****
 *
 * main(): Testa as funes ComparaDoubles() e
 *         ComparaDoubles2()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   double d;

      /* Em Matemtica, o resultado da seguinte */
      /* operao  exatamente igual a 0.1      */
   d = 10.0 - 9.9;

   /*                                         */
   /* Compara o resultado com 0.1 usando '==' */
   /*                                         */

   printf("\n>>> Usando o operador '==' <<<\n");

   if (d == 0.1) {
      printf("\n\t>>> Os valores sao iguais\n");
   } else {
      printf("\n\t>>> Os valores sao diferentes\n");
   }

   /*                                                   */
   /* Compara resultado com 0.1 usando ComparaDoubles() */
   /*                                                   */

   printf("\n\n>>> Usando ComparaDoubles() <<<\n");

   if (ComparaDoubles(d, 0.1) == 0) {
      printf("\n\t>>> Os valores sao iguais\n");
   } else {
      printf("\n\t>>> Os valores sao diferentes\n");
   }

   /*                                                    */
   /* Compara resultado com 0.1 usando ComparaDoubles2() */
   /*                                                    */

   printf("\n\n>>> Usando ComparaDoubles2() <<<\n");

   if (ComparaDoubles2(d, 0.1) == 0) {
      printf("\n\t>>> Os valores sao iguais\n");
   } else {
      printf("\n\t>>> Os valores sao diferentes\n");
   }

   return 0;
}

/****************** Resultado do Programa *****************

>>> Usando o operador '==' <<<

        >>> Os valores sao diferentes


>>> Usando ComparaDoubles() <<<

        >>> Os valores sao iguais


>>> Usando ComparaDoubles2() <<<

        >>> Os valores sao iguais

**********************************************************/
